<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class OrderMessage
 * @package Pinpoint\V12Finance\Block
 */

namespace Pinpoint\V12Finance\Block;

use Magento\Checkout\Block\Onepage\Success;
use Magento\Customer\Model\Session;
use Pinpoint\V12Finance\Helper\Data;

//@todo not too sure how implemented/used as of yet

class OrderMessage extends Success
{
    /**
     * @var string
     */
    const STATUS_PARAM_AWAITING_FULLFILLMENT = "S";

    /**
     * @var string
     */
    const STATUS_PARAM_APPROVED = "A";

    /**
     * @var string
     */
    const STATUS_PARAM_REFERRED = "R";

    /**
     * @var string
     */
    const STATUS_PARAM_DECLINED = "D";

    /**
     * @var Data
     */
    protected $_helper;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $_request;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterfaceFactory
     */
    protected $_orderFactory;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    protected $_quoteFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $_order;

    /**
     * OrderMessage constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     * @param \Magento\Framework\App\Http\Context $httpContext
     * @param Data $helper
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Pinpoint\V12Finance\Logger\Logger $logger
     * @param \Magento\Sales\Api\Data\OrderInterfaceFactory $orderFactory
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteFactory
     * @param Session $customerSession
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\Order\Config $orderConfig,
        \Magento\Framework\App\Http\Context $httpContext,
        Data $helper,
        \Magento\Framework\App\RequestInterface $request,
        \Pinpoint\V12Finance\Logger\Logger $logger,
        \Magento\Sales\Api\Data\OrderInterfaceFactory $orderFactory,
        \Magento\Quote\Api\CartRepositoryInterface $quoteFactory,
        Session $customerSession,
        array $data = []
    ) {
        parent::__construct($context, $checkoutSession, $orderConfig, $httpContext, $data);
        $this->_request = $request;
        $this->_helper = $helper;
        $this->_logger = $logger;
        $this->_orderFactory = $orderFactory;
        $this->_quoteFactory = $quoteFactory;
        $this->_checkoutSession = $checkoutSession;
        $this->_customerSession = $customerSession;
        $this->_orderConfig = $orderConfig;
    }

    /**
     * @return mixed
     */
    protected function _getApplicationStatus()
    {
        return $this->_request->getParam('status');
    }

    /**
     * @return string
     */
    protected function _getApplicationStatusText()
    {
        $appStatus = $this->_getApplicationStatus();

        switch ($appStatus) {
            case self::STATUS_PARAM_APPROVED:
            case self::STATUS_PARAM_AWAITING_FULLFILLMENT:
                return "approved";
            break;

            case self::STATUS_PARAM_DECLINED:
                return "declined";
            break;

            case self::STATUS_PARAM_REFERRED:
                return "referred";
            break;
        }
    }

    /**
     * @return mixed
     */
    protected function _getMessage()
    {
        $appStatus = $this->_getApplicationStatus();

        switch ($appStatus) {
            case self::STATUS_PARAM_AWAITING_FULLFILLMENT:
            case self::STATUS_PARAM_APPROVED:
                $message = $this->_helper->getApplicationMessage('approved');
            break;
            case self::STATUS_PARAM_REFERRED:
                $message = $this->_helper->getApplicationMessage('referred');
            break;
            case self::STATUS_PARAM_DECLINED:
            default:
                $message = $this->_helper->getApplicationMessage('declined');
            break;
        }

        return $message;
    }

    protected function _prepareLastOrder()
    {
        $orderIncrementId = $this->_request->getParam('SR');

        if ($orderIncrementId) {
            $order = $this->_orderFactory->create()->loadByIncrementId($orderIncrementId);
            $this->_order = $order;
            $orderId = $order->getId();
            if ($order->getId()) {
                // Update order payemnt with application status from V12
                $this->_getApplicationStatus();

                if ($this->_getApplicationStatus() == self::STATUS_PARAM_DECLINED) {
                    $quote = $this->_quoteFactory->create()->load($order->getQuoteId());
                    $quote->setIsActive(true)->save();
                    $this->_checkoutSession->setQuoteId($quote->getId());
                }

                $invisibleFrontStates = $this->_orderConfig->getInvisibleOnFrontStates();
                $isVisible = !in_array($order->getState(), $invisibleFrontStates);

                $this->addData([
                    'is_order_visible' => $isVisible,
                    'view_order_id' => $this->getUrl('sales/order/view/', ['order_id' => $orderId]),
                    'print_url' => $this->getUrl('sales/order/print', ['order_id'=> $orderId]),
                    'can_print_order' => $isVisible,
                    'can_view_order'  => $this->_customerSession->isLoggedIn() && $isVisible,
                    'order_id'  => $order->getIncrementId(),
                    'message' => $this->_getMessage(),
                    'is_declined' => $this->_getApplicationStatus() == self::STATUS_PARAM_DECLINED,
                    'is_approved' => $this->_getApplicationStatus() == self::STATUS_PARAM_APPROVED
                                        ||$this->_getApplicationStatus() == self::STATUS_PARAM_AWAITING_FULLFILLMENT,
                    'is_referred' => $this->_getApplicationStatus() == self::STATUS_PARAM_REFERRED,
                ]);
            }
        }

        return $this;
    }
}
