<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class Calculate
 * @package Pinpoint\V12Finance\Controller\Calculator
 */
namespace Pinpoint\V12Finance\Controller\Calculator;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;

class Calculate extends Action
{

    /**
    * @param Context $context
    */
    public function __construct(
        Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Pinpoint\V12Finance\Helper\FinancialCalculatorData $helper
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        parent::__construct($context);
    }

    public function execute()
    {
        $result = $this->resultJsonFactory->create();
        if ($this->getRequest()->isAjax()) {
            $selected_month = $this->getRequest()->getParam('selected_month', false);
            $selected_deposit = $this->getRequest()->getParam('selected_deposit', false);
            $product_price = $this->getRequest()->getParam('product_price', false);
            $numberOfMonthlyPayments = $selected_month[0];
            $depositToPay = $this->helper->calculateDepositToPay($product_price, $selected_deposit);
            $loanAmount = $this->helper->calculateLoanAmount($product_price, $depositToPay);
            $costOfLoan = $this->helper->calculateCostOfLoan($product_price, $depositToPay, $selected_month);
            $loanRepayment = $this->helper->calculateRepayment($loanAmount, $costOfLoan);
            $totalAmountPayable = $this->helper->calculateAmountPayable($depositToPay, $loanRepayment);
            $monthlyPayment = $this->helper->calculateMonthlyPrice($product_price, $selected_month, $selected_deposit);
            $data=[
                'selected_month' => implode("_",$selected_month),
                'selected_deposit' => $selected_deposit,
                'product_price' => $this->helper->getPriceFormatted($product_price),
                'deposit_to_pay' => $this->helper->getPriceFormatted($depositToPay),
                'loan_amount' => $this->helper->getPriceFormatted($loanAmount),
                'cost_of_loan' => $this->helper->getPriceFormatted($costOfLoan),
                'loan_repayment' => $this->helper->getPriceFormatted($loanRepayment),
                'total_amount_payable' => $this->helper->getPriceFormatted($totalAmountPayable),
                'monthly_payment' => $this->helper->getPriceFormatted($monthlyPayment),
                'number_of_monthly_payments' => $numberOfMonthlyPayments,
            ];
            return $result->setData($data);
        }
    }
}
