<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class Data
 * @package Pinpoint\V12Finance\Helper
 */

namespace Pinpoint\V12Finance\Helper;

use Magento\Backend\Model\Session\Quote;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\State;
use Magento\Framework\DataObjectFactory;
use Pinpoint\V12Finance\Model\System\Config\Source\MonthsOption;

class Data extends \Pinpoint\V12Finance\Helper\Config
{
    /**
     * @var string
     */
    const CONFIG_PATH_ENABLED = 'payment/ppdv12finance/active';

    /**
     * @var string
     */
    const CONFIG_PATH_MESSAGE = 'payment/ppdv12finance/messageconfig/';

    /**
     * @var string
     */
    const CONFIG_PATH_STATUS = 'payment/ppdv12finance/statusconfig/';

    /**
     * @var string
     */
    const CONFIG_PATH_MIN_DEPOSIT = 'payment/ppdv12finance/minimum_deposit_amount';

    /**
     * @var string
     */
    const CONFIG_PATH_MAX_DEPOSIT = 'payment/ppdv12finance/maximum_deposit_amount';

    /**
     * @var string
     */
    const CONFIG_PATH_VARIABLE_DEPOSIT = 'payment/ppdv12finance/variable_deposit';

    /**
     * @var string
     */
    const CONFIG_PATH_MIN_ORDER = 'payment/ppdv12finance/minimum_order_amount';

    /**
     * @var string
     */
    const CONFIG_PATH_ALL_PRODUCTS = 'payment/ppdv12finance/all_products';

    /**
     * @var string
     */
    const CONFIG_MONTHLY_OPTIONS = 'payment/ppdv12finance/manageproducts/months_range';

    /**
     * @var MonthsOption
     */
    protected $_monthOptions;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \Magento\Backend\Model\Session\Quote
     */
    protected $_adminQuote;

    /**
     * Data constructor.
     * @param State $state
     * @param Quote $adminQuote
     * @param DataObjectFactory $dataObjectFactory
     * @param Context $context
     * @param MonthsOption $monthsOption
     */
    public function __construct(
        State $state,
        Quote $adminQuote,
        DataObjectFactory $dataObjectFactory,
        Context $context,
        MonthsOption $monthsOption
    ) {
        $this->_state = $state;
        $this->_adminQuote = $adminQuote;
        $this->_monthOptions = $monthsOption;
        parent::__construct(
            $dataObjectFactory,
            $context
        );
    }

    /**
     * @param null $store
     * @return bool|mixed
     */
    public function isEnabled($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_ENABLED,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param null $store
     * @return mixed
     */
    public function enabledAllProducts($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_ALL_PRODUCTS,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param $key
     * @return mixed
     */
    public function getApplicationMessage($key)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_MESSAGE . $key,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            null
        );
    }

    /**
     * @param null $store
     * @return mixed
     */
    public function getVariableDeposit($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_VARIABLE_DEPOSIT,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param null $store
     * @return mixed
     */
    public function getMinDepositPercentage($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_MIN_DEPOSIT,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param null $store
     * @return mixed
     */
    public function getMaxDepositPercentage($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_MAX_DEPOSIT,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param null $store
     * @return mixed
     */
    public function getMinOrderAmount($store = null)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_MIN_ORDER,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param $key
     * @return mixed
     */
    public function getApplicationStatus($key)
    {
        return $this->scopeConfig->getValue(
            self::CONFIG_PATH_STATUS . $key,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            null
        );
    }

    /**
     * @param $price
     * @return string
     */
    public function formatPrice($price)
    {
        return number_format($price, 2, '.', '');
    }

    /**
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isAdmin()
    {
        return ('adminhtml' == $this->_state->getAreaCode());
    }

    /*
     * @return array
     */
    public function getSelectedProducts()
    {
        $list = $this->scopeConfig->getValue(self::CONFIG_MONTHLY_OPTIONS, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $list = explode(',', $list);
        $monthOptions = $this->_monthOptions->getOriginalOption();

        foreach ($monthOptions as $key => $value) {
            if (in_array($key, $list)) {
                $result[] = $key;
            }
        }
        return $result;
    }
}
