<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class ApiAbstract
 * @package Pinpoint\V12Finance\Model\Rest\Api
 */

namespace Pinpoint\V12Finance\Model\Rest\Api;

use Magento\Framework\Serialize\Serializer\Json;
use Pinpoint\V12Finance\Helper\Data as V12Helper;
use Pinpoint\V12Finance\Logger\Logger as Logger;

class ApiAbstract
{
    /**
     * @var Logger
     */
    protected $_logger;

    /**
     * @var V12Helper
     */
    protected $_helper;

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    protected $curl;

    /**
     * @var Json
     */
    protected $json;

    /**
     * ApiAbstract constructor.
     * @param V12Helper $helper
     * @param Logger $logger
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param Json $json
     */
    public function __construct(
        V12Helper $helper,
        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        Json $json
    ) {
        $this->_helper = $helper;
        $this->_logger = $logger;
        $this->curl = $curl;
        $this->json = $json;
    }

    /**
     * @param $requestUrl
     * @param array $params
     * @return array|bool|float|int|mixed|string|null
     */
    protected function request($requestUrl, $params = [])
    {
        $curl = $this->curl;

        $curl->setHeaders([
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
        ]);

        $curl->setOptions([
            \CURLOPT_RETURNTRANSFER => 1,
            \CURLOPT_ENCODING => '',
            \CURLOPT_MAXREDIRS => 0,
            \CURLOPT_TIMEOUT => 30
        ]);

        $curl->post($requestUrl, $this->json->serialize($params));

        $response = $this->json->unserialize($curl->getBody());

        if ($this->apiErrorCheck($response)) {
            return [];
        }

        return $response;
    }

    /**
     * @param $response
     * @return bool|string
     */
    protected function apiErrorCheck($response)
    {
        if (!empty($response['Errors'])) {
            $errmessage = '';
            foreach ($response['Errors'] as $error) {
                $errmessage .= '[' . $error['Code'] . '] ' . $error['Description'];
            }

            return $errmessage;
        }

        return false;
    }
}
