<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class ApplicationRequest
 * @package Pinpoint\V12Finance\Model\Rest\Api
 */

namespace Pinpoint\V12Finance\Model\Rest\Api;

use Magento\Framework\Serialize\Serializer\Json;
use Pinpoint\V12Finance\Helper\Data as V12Helper;
use Pinpoint\V12Finance\Logger\Logger as Logger;
use Pinpoint\V12Finance\Model\Rest\Api\Customer as ApiCustomer;
use Pinpoint\V12Finance\Model\Rest\Api\Order as ApiOrder;

class ApplicationRequest extends ApiAbstract
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_psrlogger;

    /**
     * @var Customer
     */
    protected $_apiCustomer;

    /**
     * @var Order
     */
    protected $_apiOrder;

    /**
     * ApplicationRequest constructor.
     * @param Order $apiOrder
     * @param Customer $apiCustomer
     * @param V12Helper $helper
     * @param Logger $logger
     * @param \Psr\Log\LoggerInterface $psrlogger
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param Json $json
     */
    public function __construct(
        ApiOrder $apiOrder,
        ApiCustomer $apiCustomer,
        V12Helper $helper,
        Logger $logger,
        \Psr\Log\LoggerInterface $psrlogger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        Json $json
    ) {
        parent::__construct($helper, $logger, $curl, $json);
        $this->_apiOrder = $apiOrder;
        $this->_apiCustomer = $apiCustomer;
        $this->_psrlogger = $psrlogger;
    }

    /**
     * @param $order
     * @return array|bool|float|int|mixed|string|null
     */
    public function makeApplicationRequest($order)
    {
        $requestUrl = $this->_helper->getGatewayUrl('SubmitApplication');
        $params = $this->_getApplicationRequestData($order);

        $response = $this->request($requestUrl, $params);

        //check for errors
        $this->apiErrorCheck($response);

        return $response;
    }

    /**
     * @param Mage_Sales_Model_Order $order
     * @return mixed
     */
    protected function _getApplicationRequestData($order)
    {
        $appRequest             = [];
        $appRequest['customer'] = $this->_apiCustomer->fromMageCustomer($order);
        $appRequest['order']    = $this->_apiOrder->fromMageOrder($order);
        $appRequest['retailer'] = $this->_helper->getRetailerAuth();

        return $appRequest;
    }
}
