<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class SubmitSuccess
 * @package Pinpoint\V12Finance\Observer\Sales\Quote
 */

namespace Pinpoint\V12Finance\Observer\Sales\Quote;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Message\ManagerInterface;
use Pinpoint\V12Finance\Helper\Data;
use Pinpoint\V12Finance\Logger\Logger;

class SubmitSuccess implements ObserverInterface
{

    /**
     * @var \Pinpoint\V12Finance\Logger\Logger
     */
    protected $_logger;

    /**
     * @var \Pinpoint\V12Finance\Helper\Data
     */
    protected $_helper;

    /**
     * @var \Magento\Backend\Model\Session
     */
    protected $_backendSession;

    /**
     * @var ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    protected $orderRepository;

    /**
     * SubmitSuccess constructor.
     * @param ManagerInterface $messageManager
     * @param Data $_helper
     * @param Logger $logger
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     */
    public function __construct(
        ManagerInterface $messageManager,
        Data $_helper,
        Logger $logger,
        \Magento\Sales\Model\OrderRepository $orderRepository
    ) {
        $this->_logger = $logger;
        $this->_helper = $_helper;
        $this->_messageManager = $messageManager;
        $this->orderRepository = $orderRepository;
    }

    /**
     * @param Observer $observer
     * @return $this
     */
    public function execute(Observer $observer)
    {
        /** @var \Magento\Sales\Model\Order $order */
        $order = $observer->getOrder();

        if ($order->getPayment()->getMethod() === \Pinpoint\V12Finance\Model\Payment\Method\V12Finance::CODE) {
            try {
                if ($state = $this->_helper->getApplicationStatus('incomplete')) {
                    $order->setStatus($state);
                    $order->setState($state);

                    $this->orderRepository->save($order);
                }
            } catch (\Exception $e) {
                $this->_messageManager->addErrorMessage(__('The default V12 Magento status could not be set.'));
            }
        }

        return $this;
    }
}
