<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Carmine D'Amore <carmine@pinpointdesigns.co.uk>
 * Code Release Version: 1.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class InstallSchema
 * @package Pinpoint\V12Finance\Setup
 */

namespace Pinpoint\V12Finance\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{
    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();

        //V12 finance products table
        $productTableName = $installer->getTable('ppd_v12finance_product');
        $productXrefTableName = $installer->getTable('ppd_v12finance_product_xref');
        if (!$installer->tableExists($productTableName)) {
            $table_v12_product = $installer->getConnection()->newTable(
                $productTableName
            )->addColumn(
                'id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                [
                    'identity' => true,
                    'nullable' => false,
                    'primary' => true,
                    'unsigned' => true,
                ],
                'Unique table ID'
            )->addColumn(
                'product_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false
                ],
                'Finance product id'
            )->addColumn(
                'guid',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                [
                    'nullable' => false
                ],
                'Finance product guid'
            )->addColumn(
                'name',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                128,
                [
                    'nullable' => false
                ],
                'Product name'
            )->addColumn(
                'tag',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                128,
                [
                    'nullable' => false
                ],
                'product short name'
            )->addColumn(
                'months',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                4,
                [
                    'nullable' => false
                ],
                'Month total finance runs for (12,24,36...)'
            )->addColumn(
                'monthly_rate',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'Monthly rate of interest'
            )->addColumn(
                'max_loan',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'The maximum loan value permitted for this product.'
            )->addColumn(
                'min_loan',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'The minimum loan value permitted for this product.'
            )->addColumn(
                'calc_factor',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,4',
                    'nullable' => false,
                    'default' => 0.0000
                ],
                'A factor that can be used to calculate finance payments.'
            )->addColumn(
                'apr',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'APR rate.'
            )->addColumn(
                'option_period',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'The option period (in months) that apply to this product (if applicable).'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                null,
                [
                    'nullable' => false
                ],
                'Product description'
            )->addColumn(
                'service_fee',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'Service fee if applicable'
            )->addColumn(
                'document_fee',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'Document fee, as a flat rate, for the product (if applicable)'
            )->addColumn(
                'document_fee_collection_month',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                16,
                [
                    'nullable' => false
                ],
                'What month in the life of the loan will the document fee be collected.'
            )->addColumn(
                'document_fee_maximum',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                16,
                [
                    'nullable' => false,
                    'default' => 0
                ],
                'The maximum document fee permitted for this product.'
            )->addColumn(
                'document_fee_minimum',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                16,
                [
                    'nullable' => false,
                    'default' => 0
                ],
                'The minimum document fee permitted for this product.'
            )->addColumn(
                'document_fee_percentage',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                16,
                [
                    'nullable' => false,
                    'default' => 0
                ],
                'Document fee, as a percentage of loan value, for the product (if applicable).'
            )->addColumn(
                'deferred_period',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                16,
                [
                    'nullable' => false
                ],
                'Only applies to Buy Now Pay Later (BNPL) products. This is the number of months before the first payment is due.'
            )->addColumn(
                'settlement_fee',
                \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                16,
                [
                    'length' => '10,2',
                    'nullable' => false,
                    'default' => 0.00
                ],
                'The settlement fee (if applicable) for this product.'
            )->addColumn(
                'active',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                1,
                [
                    'nullable' => false,
                    'unsigned' => true,
                    'default' => 1
                ],
                'Finance product active status'
            )->addColumn(
                'created_at',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT],
                'Created At'
            )->addColumn(
                'updated_at',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE],
                'Updated At'
            );
            $installer->getConnection()->createTable($table_v12_product);

            //V12 finance product -> magento product xref table
            $table_v12_product_xref = $installer->getConnection()->newTable(
                $productXrefTableName
            )->addColumn(
                'v12product_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true,
                ],
                'v12 finance product ID'
            )->addColumn(
                'product_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true,
                ],
                'Magento product ID'
            )->addForeignKey(
                $installer->getFkName(
                    $productXrefTableName,
                    'v12product_id',
                    $productTableName,
                    'id'
                ),
                'v12product_id',
                $productTableName,
                'id',
                \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
            )->addForeignKey(
                $installer->getFkName(
                    $productXrefTableName,
                    'product_id',
                    $installer->getTable('catalog_product_entity'),
                    'entity_id'
                ),
                'product_id',
                $installer->getTable('catalog_product_entity'),
                'entity_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
            );
            $installer->getConnection()->createTable($table_v12_product_xref);
        }

        $installer->endSetup();
    }
}
